@@ -16,6 +16,13 @@ from ..utils import YieldSearch, Writer
 
 class Veloce(NodeVisitor):
 
+    @classmethod
+    def translate(cls, code):
+        translator = cls()
+        ast = parse(code)
+        translator.visit(ast)
+        return translator.writer.value()
+
     def __init__(self):
         super().__init__()
         self.dependencies = []
@@ -77,21 +84,21 @@ class Veloce(NodeVisitor):
     ######################################################################
 
     # Interactive(stmt* body)
-    visit_Interactive = NotImplemented
+    def visit_Interactive(self, node): raise NotImplemented
 
     # Expression(expr body)
-    visit_Expression = NotImplemented
+    def visit_Expression(self, node): raise NotImplemented
 
     # Suite(stmt* body)
-    visit_Suite = NotImplemented
+    def visit_Suite(self, node): raise NotImplemented
 
     # expr_context = Load | Store | Del | AugLoad | AugStore | Param
-    visit_Load = NotImplemented
-    visit_Store = NotImplemented
-    visit_Del = NotImplemented
-    visit_AugLoad = NotImplemented
-    visit_AugStore = NotImplemented
-    visit_Param = NotImplemented
+    def visit_Load(self, node): raise NotImplemented
+    def visit_Store(self, node): raise NotImplemented
+    def visit_Del(self, node): raise NotImplemented
+    def visit_AugLoad(self, node): raise NotImplemented
+    def visit_AugStore(self, node): raise NotImplemented
+    def visit_Param(self, node): raise NotImplemented
 
     # Pass
     def visit_Pass(self, node):
@@ -123,14 +130,14 @@ class Veloce(NodeVisitor):
         return 'yield {}'.format(self.visit(node.value))
 
     # YieldFrom(expr value)
-    visit_YieldFrom = NotImplemented
+    def visit_YieldFrom(self, node): raise NotImplemented
 
     # In
     def visit_In(self, node):
         return ' in '
 
     # NotIn
-    visit_NotIn = NotImplemented
+    def visit_NotIn(self, node): raise NotImplemented
 
     # Module(stmt* body)
     def visit_Module(self, node):
@@ -145,13 +152,13 @@ class Veloce(NodeVisitor):
         return '[{}]'.format(', '.join(map(self.visit, node.elts)))
 
     # Set(expr* elts)
-    visit_Set = NotImplemented
+    def visit_Set(self, node): raise NotImplemented
 
     # alias = (identifier name, identifier? asname)
-    visit_alias = NotImplemented
+    def visit_alias(self, node): raise NotImplemented
 
     # Import(alias* names)
-    visit_Import = NotImplemented
+    def visit_Import(self, node): raise NotImplemented
 
     # ImportFrom(identifier? module, alias* names, int? level)
     def visit_ImportFrom(self, node):
@@ -184,7 +191,7 @@ class Veloce(NodeVisitor):
         return ''
 
     # Nonlocal(identifier* names)
-    visit_Nonlocal = NotImplemented
+    def visit_Nonlocal(self, node): raise NotImplemented
 
     # FunctionDef(identifier name, arguments args, stmt* body, expr* decorator_list, expr? returns)
     def visit_FunctionDef(self, node):
@@ -291,7 +298,7 @@ class Veloce(NodeVisitor):
         return self.visit(node.value)
 
     # ExtSlice(slice* dims) 
-    visit_ExtSlice = NotImplemented
+    def visit_ExtSlice(self, node): raise NotImplemented
 
     # Subscript(expr value, slice slice, expr_context ctx)
     def visit_Subscript(self, node):
@@ -311,7 +318,7 @@ class Veloce(NodeVisitor):
         return args, kwargs, vararg, varkwargs
 
     # arg = (identifier arg, expr? annotation)
-    visit_arg = NotImplemented
+    def visit_arg(self, node): raise NotImplemented
 
     # Name(identifier id, expr_context ctx)
     def visit_Name(self, node):
@@ -386,22 +393,34 @@ class Veloce(NodeVisitor):
             s = 'console.log({})'.format(', '.join(args))
             return s
         else:
+            # positional args
             if node.args:
                 args = [self.visit(e) for e in node.args]
                 args = [e for e in args if e]
             else:
                 args = []
-            if node.keywords and not node.kwargs:
-                keywords = '__kwargs{}__'.format(self.uuid())
-                _ = ', '.join(map(lambda x: '{}: {}'.format(x[0], x[1]), map(self.visit, node.keywords)))
-                self.writer.write('var {} = {{{}}}'.format(keywords, _))
-                args.append(keywords)
-            elif node.kwargs and not node.keywords:
-                args.append(node.kwargs.id)
-            elif node.kwargs and node.keywords:
+            # variable arguments aka. starargs
+            call_arguments = 'call_arguments{}'.format(self.uuid())
+            if node.starargs:
+                varargs = self.visit(node.starargs)
+                code = "for i in {}: jscode('{}.push(i)')".format(varargs, call_arguments)
+                self.writer.write(self.translate(code))
+            # keywords and variable keywords arguments aka. starkwargs
+            if node.kwargs:
+                kwargs = self.visit(node.kwargs)
+                if node.keywords:
+                    for key, value in map(self.visit, node.keywords):
+                        self.writer.write('{}["{}"] = {};'.format(kwargs, key, value))
+            elif node.keywords:
+                kwargs = '__pythonium_kwargs'
+                self.writer.write('var __pythonium_kwargs = {};')
                 for key, value in map(self.visit, node.keywords):
-                    self.writer.write('{}.{} = {}'.format(node.kwargs.id, key, value))
-                args.append(node.kwargs.id)
+                    self.writer.write('{}["{}"] = {};'.format(kwargs, key, value))
+            if node.kwargs or node.keywords:
+                # XXX: define for every call since we can't define it globaly
+                self.writer.write('__ARGUMENTS_PADDING__ = {};')
+                args.append('__ARGUMENTS_PADDING__')
+                args.append(kwargs)
             return '{}({})'.format(name, ', '.join(args))
 
     # ListComp(expr elt, comprehension* generators)
@@ -638,7 +657,20 @@ class Veloce(NodeVisitor):
             self.writer.write('}')
 
     # IfExp(expr test, expr body, expr orelse)
-    visit_IfExp = NotImplemented
+    def visit_IfExp(self, node):
+        name = '__pythonium_ifexp_{}'.format(self.uuid())
+        self.writer.write('if ({}) {{'.format(self.visit(node.test)))
+        self.writer.push()
+        body = self.visit(node.body)
+        self.writer.write('var {} = {};'.format(name, body))
+        self.writer.pull()
+        self.writer.write('} else {')
+        self.writer.push()
+        orelse = self.visit(node.orelse)
+        self.writer.write('var {} = {};'.format(name, orelse))
+        self.writer.pull()
+        self.writer.write('}')
+        return name
 
     # Ellipsis
     visit_Ellipsis = NotImplemented
@@ -686,10 +718,59 @@ class Veloce(NodeVisitor):
     def visit_Break(self, node):
         self.writer.write('break;')
 
+    def _unpack_arguments(self, args, kwargs, varargs, varkwargs):
+        self.writer.write('/* BEGIN arguments unpacking */')
+        if not varkwargs and kwargs:
+            varkwargs = '__kwargs'
+
+        if varargs or (varkwargs and varkwargs != '__kwargs') or kwargs:
+            self.writer.write('var __args = Array.prototype.slice.call(arguments);')
+        if (varkwargs and varkwargs != '__kwargs') or kwargs:
+            self.writer.write('if (__args[__args.length - 2] === __ARGUMENTS_PADDING__) {')
+            self.writer.push()
+            self.writer.write('var {} = __args[__args.length - 1];'.format(varkwargs))
+            self.writer.write('var varkwargs_start = __args.length - 2;')
+            self.writer.pull()
+            self.writer.write('} else {')  # no variable keywords was provided so it's empty
+            self.writer.push()
+            self.writer.write('var {} = {{}};'.format(varkwargs))
+            self.writer.write('var varkwargs_start = undefined;')
+            self.writer.pull()
+            self.writer.write('}')
+        num_args = len(args)
+        for index, keyword in enumerate(kwargs.keys()):
+            position = num_args + index - 1
+            self.writer.write('if (varkwargs_start !== undefined && {} > varkwargs_start) {{'.format(position))
+            self.writer.push()
+ 
+            self.writer.write('{} = {}[{}] || {};'.format(keyword, varkwargs, keyword, kwargs[keyword])) 
+            self.writer.pull()
+            self.writer.write('} else {')
+            self.writer.push()
+            self.writer.write('{} = {} || {}[{}] || {};'.format(keyword, keyword, varkwargs, keyword, kwargs[keyword]))
+            self.writer.pull()
+            self.writer.write('}')
+            if varkwargs != '__kwargs':
+                self.writer.write('delete {}.{};'.format(varkwargs, keyword))
+        if varargs:
+            self.writer.write('__args = __args.splice({});'.format(len(args)))
+            if varkwargs and (varkwargs != '__kwargs' or kwargs):
+                self.writer.write('if (varkwargs_start) {{ __args.splice(varkwargs_start - {}) }}'.format(len(args)))
+            self.writer.write('{} = __args;'.format(varargs))
+        self.writer.write('/* END arguments unpacking */')
+
     # Lambda(arguments args, expr body)
     def visit_Lambda(self, node):
-        args = ', '.join(map(self.visit, node.args.args))
-        return '(function ({}) {{{}}})'.format(args, self.visit(node.body))
+        args, kwargs, vararg, varkwargs = self.visit(node.args)
+        name = '__lambda{}'.format(self.uuid())
+        self.writer.write('var {} = function({}) {{'.format(name, ', '.join(args)))
+        self.writer.push()
+        self._unpack_arguments(args, kwargs, vararg, varkwargs)
+        body = self.visit(node.body)
+        self.writer.write(body)
+        self.writer.pull()
+        self.writer.write('}')
+        return name
 
     # ClassDef(identifier name, expr* bases, keyword* keywords, 
     #          expr? starargs, expr? kwargs, stmt* body, expr* decorator_list)
